//
// Giuseppe Vacanti, XMM-SOC ESA, 1998
//

// C++
#include <string>
#include <cerrno>
#include <cstdio>

// C
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>

// SAS
#include "Tracer.h"
#include "errstr.h"

// RSA MD5
#include "md5global.h"
#include "md5.h"

// This application
#include "DirEntryImplementation.h"

DirEntryImplementation::DirEntryImplementation ()
  : _name("."),
    _isSigned(false)
{
  PROC ENDPROC
}

DirEntryImplementation::DirEntryImplementation (const string name)
  : _name(name),
    _isSigned(false)
{
  PROC
  ENDPROC
}

string DirEntryImplementation::name() const {
  PROC
  RETURN _name;
}

string DirEntryImplementation::dirname() const {
  PROC
    // 1999-07-26: fudge no longer needed in egcs 1.1.1
    // fudge to work around a g++ bug
    //  string::size_type k = 0;
    // for(string::size_type i = 0 ; i < _name.length(); i++) {
    // if(_name[i] == '/')
    //  k=i;
    // }
  string::size_type i = _name.find_last_of("/");
  string s = _name.substr(0,i+1);
  //   string s = _name.substr(0,k+1);
  RETURN s;
}

string DirEntryImplementation::basename() const {
  PROC
    // 1999-07-26: fudge no longer needed in egcs 1.1.1
    // fudge to work around a g++ bug
    //  string::size_type k = 0;
//   for(string::size_type i = 0; i < _name.length(); i++) {
//     if(_name[i] == '/')
//       k=i;
//   }
//   string s = _name.substr(k+1,_name.length() - 1);
  string::size_type i = _name.find_last_of("/");
  string s = _name.substr(i+1);
  RETURN s;
    }

off_t DirEntryImplementation::size() const {
  PROC
  errno = 0;
  struct stat buf;  
  if (stat(_name.c_str(), &buf) == 0) {
    if(errno) {
      errstr << "Error " << errno << " on " + _name << Sas::warning("size");
      errno = 0;
    }
    ALTRETURNV(buf.st_size)
  } else {
    RETURN 0;
  }
}

bool DirEntryImplementation::isDir() const {
  PROC
  struct stat buf;
  errno = 0;
  if (stat(_name.c_str(), &buf) == 0) {
    if(errno) {
      errstr << "Error " << errno << " on " + _name << Sas::warning("isDir");
      errno = 0;
    }
    ALTRETURNV(S_ISDIR(buf.st_mode))
  } else {
    RETURN false;
  }
}


void DirEntryImplementation::sign() {
  PROC
  FILE *file;
  const size_t buffer_length = 1024;
  unsigned char buffer[buffer_length];
  if ((file = fopen(name().c_str(), "rb")) != NULL) {
    MD5_CTX context;
    MD5Init (&context);
    while(size_t len = fread(buffer, 1, buffer_length, file))
      MD5Update (&context, buffer, len);
    unsigned char digest[16];
    MD5Final (digest, &context);
    fclose (file);
    _isSigned = true;
    char sp[33];
    for (int i = 0; i < 16; i++) {
      sprintf(&sp[2*i], "%02x", digest[i]);
    }
    _signature = sp;
  } else {
    errstr << name() << " cannot be digitally signed. " <<
      Sas::warning("digitalSignature");
  }
  ENDPROC
}

string DirEntryImplementation::signature() const {
  PROC
  if(! isSigned())
    errstr << name() << " is not digitally signed yet." <<
      Sas::warning("digitalSignature");
  RETURN _signature;
}

bool DirEntryImplementation::isSigned() const {
  PROC
  RETURN _isSigned;
}

void DirEntryImplementation::print(ostream & os) const
{
  PROC
    os << name() + ", " << size() << " bytes";
  if(isSigned())
    os << ", " + signature();
  ENDPROC
}

bool DirEntryImplementation::operator==(const DirEntry & rhs) const
{
  PROC
  if(isSigned() && rhs.isSigned())
    ALTRETURNV(name() == rhs.name() && size() == rhs.size() && signature() == rhs.signature())
  else
    RETURN name() == rhs.name() && size() == rhs.size();
  ENDPROC
}

bool DirEntryImplementation::operator!=(const DirEntry & rhs) const
{
  PROC
  RETURN ! (*this == rhs);
}
