//
// XMM Science Analysis System
//
// Giuseppe Vacanti (European Space Agency) 
// May 03, 2000
//

// This runs on the output of testevatt.f90, and checks that the
// values set through the f90 interface appear as they should in C++

#include <EventAttributes.h>
#include <iostream>
#include <errstr.h>
#include <vector>
#include <string>
#include <cassert>

int main()
{
  using namespace EventAttributes;
  DataSet *set = dataSetServer->open("evatt.ds", DataSet::Read);
  Table *tab = set->table("evatt");
  vector<string> ea = names();
  // The table must have all the attributes that could be set
  bool fail = false;
  for(unsigned i = 0; i < ea.size(); i++){
    cout << "Looking for " + ea[i];
    if(!eventAttributeIsSet(tab,lookup(ea[i]))){
      fail = true;
      cout << " NO!" << endl;
    } else {
      cout << " ok" << endl;
    }
  }

  //  Column *c1 = tab->column("Attribute_Name");
  Column *c2 = tab->column("FLAG");

  ColumnData *cd2 = c2->data();  
  int32 *data = cd2->int32Data();

  for(unsigned i = 0; i < ea.size(); i++){
    cout << "Name = " + ea[i] << " value set = " <<  data[i] << " bit number = " << static_cast<unsigned>(lookup(ea[i])) << endl;
    int v = 0;
    setEventAttribute(v, lookup(ea[i]));
    assert(data[i] == v);
  }
  delete cd2;

  
  cout << "Checking consistency of rejection masks f90 vs c++" << endl;
  long rgs_msk = tab->attribute("RGS_MSK")->asInt();
  cout << "RGS rejection mask f90/c++ " << rgs_msk << " "
       << RGS_REJECTION_MASK;
  if(rgs_msk != RGS_REJECTION_MASK){
    fail = true;
    cout << " FAIL";
  }
  cout << endl;

  long emos_msk = tab->attribute("EMOS_MSK")->asInt();
  cout << "EMOS rejection mask f90/c++ " <<emos_msk << " "
       << EMOS_REJECTION_MASK;
  if(emos_msk != EMOS_REJECTION_MASK){
    fail = true;
    cout << " FAIL";
  }
  cout << endl;

  long epn_msk = tab->attribute("EPN_MSK")->asInt();
  cout << "EPN rejection mask f90/c++ " << epn_msk << " "
       << EPN_REJECTION_MASK;
  if(epn_msk != EPN_REJECTION_MASK){
    fail = true;
    cout << " FAIL";
  }
  cout << endl;

  dataSetServer->close(set);

  assert(!fail);

}
