#include <EventAttributes.h>
#include <iostream>
#include <errstr.h>
#include <vector>
#include <string>
#include <cassert>
#include <algorithm>
#include <sstream>

#include <DataFilter.h>


namespace {
  class PrintRange {
  public:
    void operator()(const DataFilter::RowRange& r){
      std::cout << r.startRow << " to " << r.stopRow << std::endl;
    }
  };

}



int main()
{
  using namespace EventAttributes;
  vector<string> ea = names();
  for(unsigned int i = 0; i < ea.size(); i++)
    cout << "The event attribute " + ea[i] + " maps onto bit "
	 << (int) lookup(ea[i]) << endl;

  // There are 41 distinct event attributes
  assert(ea.size() == 41);
  
  DataSet *set = dataSetServer->open("testevattcc.ds", DataSet::Create);
  *(set->addAttribute("INSTRUME")) = "EMOS1";
  Table *tab = set->addTable("FROM", ea.size());
  Column *col = tab->addColumn(COLUMN_NAME, COLUMN_TYPE);
  ColumnData *ptr = col->data();
  int32 *v = ptr->int32Data();
  for(unsigned i = 0; i < ea.size(); ++i){
    v[i] = CLEAR_ALL_FLAGS;
    setEventAttribute(v[i], lookup(ea[i]));
    std::cout << i << " " << v[i] << " " << lookup(ea[i]) << " " << ea[i] << std::endl;
    setEventAttribute(tab, lookup(ea[i]));
  }

  // expected number of matching rows (calculated by hand by 
  // looking at EventAttributes.h.
  std::vector<unsigned> V(32);
   V[0] = 2;
   V[1] = 1;
   V[2] = 1;
   V[3] = 3;
   V[4] = 2;
   V[5] = 1;
   V[6] = 1;
   V[7] = 1;
   V[8] = 1;
   V[9] = 1;
  V[10] = 1;
  V[11] = 1;
  V[12] = 2;
  V[13] = 1;
  V[14] = 0;
  V[15] = 1;
  V[16] = 1;
  V[17] = 1;
  V[18] = 1;
  V[19] = 2;
  V[20] = 2;
  V[21] = 1;
  V[22] = 2;
  V[23] = 2;
  V[24] = 1;
  V[25] = 2;
  V[26] = 1;
  V[27] = 1;
  V[28] = 1;
  V[29] = 1;
  V[30] = 1;
  V[31] = 1;

  for(unsigned i = 0; i < ea.size(); ++i){
    stringstream s;
    writeSelectExpression(s, lookup(ea[i]));
    DataFilter f(s.str());
    std::cout << "Testing whether " << ea[i]
	      << " is set with the following expression: "
	      << s.str() << std::endl;
    std::vector<DataFilter::RowRange> rr;
    f.doSelect(*tab, rr);
    int l = static_cast<int>(lookup(ea[i]));
    unsigned matching = 0;
    for(unsigned m = 0; m < rr.size(); ++m){
      matching += rr[m].stopRow - rr[m].startRow + 1;
    }
    std::cout << matching  << " matching rows (expect " << V[l] << ") :\n";
    std::for_each(rr.begin(), rr.end(), PrintRange());
    

    assert(matching == V[l]);
  }
  
  bool fail = false;
  for(unsigned i = 0; i < ea.size(); i++){
    if(!eventAttributeIsSet(tab,lookup(ea[i]))){
      fail = true;
      Sas::warning("eventAttributeNotSet", ea[i]);
    }
    //    tab->deleteAttribute(name);
  }
  if(fail)
    exit(1);

  setEmosRejectionMask(tab);
  setEpnRejectionMask(tab);
  setRgsRejectionMask(tab);
  
  // SCR 198
  setEmosSpectralMask(tab);

  Table *tab1 = set->addTable("TO",0);
  copyEventAttributes(tab,tab1);
  for(unsigned i = 0; i < ea.size(); i++){
    if(!eventAttributeIsSet(tab1,lookup(ea[i]))){
      fail = true;
      Sas::warning("eventAttributeNotPropagated", ea[i]);
    }
  }
  if(fail)
    exit(1);
  
  // Create he following entries like this:
  //
  //./testevattcc  | awk '{printf("\tassert(EventAttributes::%s == lookup(\"%s\"));\n",$4,$4);}'
  //

  assert(EventAttributes::BAD_E3E4 == lookup("BAD_E3E4"));
  assert(EventAttributes::BAD_SHAPE == lookup("BAD_SHAPE"));
  assert(EventAttributes::BELOW_ACCEPTANCE == lookup("BELOW_ACCEPTANCE"));
  assert(EventAttributes::CLOSE_TO_BADCOL == lookup("CLOSE_TO_BADCOL"));
  assert(EventAttributes::CLOSE_TO_BADROW == lookup("CLOSE_TO_BADROW"));
  assert(EventAttributes::CLOSE_TO_BRIGHTPIX == lookup("CLOSE_TO_BRIGHTPIX"));
  assert(EventAttributes::CLOSE_TO_CCD_BORDER == lookup("CLOSE_TO_CCD_BORDER"));
  assert(EventAttributes::CLOSE_TO_CCD_WINDOW == lookup("CLOSE_TO_CCD_WINDOW"));
  assert(EventAttributes::CLOSE_TO_DEADPIX == lookup("CLOSE_TO_DEADPIX"));
  assert(EventAttributes::CLOSE_TO_FLICKERINGPIX == lookup("CLOSE_TO_FLICKERINGPIX"));
  assert(EventAttributes::CLOSE_TO_NODE_BOUNDARY == lookup("CLOSE_TO_NODE_BOUNDARY"));
  assert(EventAttributes::CLOSE_TO_ONBOARD_BADPIX == lookup("CLOSE_TO_ONBOARD_BADPIX"));
  assert(EventAttributes::COSMIC_RAY == lookup("COSMIC_RAY"));
  assert(EventAttributes::DIAGONAL == lookup("DIAGONAL"));
  assert(EventAttributes::EVLIST_LOCAL == lookup("EVLIST_LOCAL"));
  assert(EventAttributes::IN_BAD_FRAME == lookup("IN_BAD_FRAME"));
  assert(EventAttributes::IN_SPOILED_FRAME == lookup("IN_SPOILED_FRAME"));
  assert(EventAttributes::MIP_ASSOCIATED == lookup("MIP_ASSOCIATED"));
  assert(EventAttributes::NEXT_TO_BADPIX == lookup("NEXT_TO_BADPIX"));
  assert(EventAttributes::NEXT_TO_CCD_BORDER == lookup("NEXT_TO_CCD_BORDER"));
  assert(EventAttributes::NEXT_TO_OFFSET_COLUMN == lookup("NEXT_TO_OFFSET_COLUMN"));
  assert(EventAttributes::ONBOARD_SER == lookup("ONBOARD_SER"));
  assert(EventAttributes::ON_BADPIX == lookup("ON_BADPIX"));
  assert(EventAttributes::ON_BADROW == lookup("ON_BADROW"));
  assert(EventAttributes::ON_NODE0 == lookup("ON_NODE0"));
  assert(EventAttributes::ON_NODE1 == lookup("ON_NODE1"));
  assert(EventAttributes::ON_OFFSET_COLUMN == lookup("ON_OFFSET_COLUMN"));
  assert(EventAttributes::OUTSIDE_GTI == lookup("OUTSIDE_GTI"));
  assert(EventAttributes::OUTSIDE_THRESHOLDS == lookup("OUTSIDE_THRESHOLDS"));
  assert(EventAttributes::OUT_OF_CCD_WINDOW == lookup("OUT_OF_CCD_WINDOW"));
  assert(EventAttributes::OUT_OF_FOV == lookup("OUT_OF_FOV"));
  assert(EventAttributes::REJECTED_BY_GATTI == lookup("REJECTED_BY_GATTI"));
  assert(EventAttributes::SECONDARY == lookup("SECONDARY"));
  assert(EventAttributes::TRAILING == lookup("TRAILING"));
  assert(EventAttributes::UNDERSHOOT == lookup("UNDERSHOOT"));
  assert(EventAttributes::UNKNOWN_RAWX_PATTERN == lookup("UNKNOWN_RAWX_PATTERN"));
  assert(EventAttributes::FLICKERING == lookup("FLICKERING"));
  assert(EventAttributes::ON_BADOFFSET == lookup("ON_BADOFFSET"));

  delete ptr;
  dataSetServer->close(set);
}
